@@ -3,7 +3,7 @@
 /*
     This file is part of Magnum.
 
-    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018
+    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019
               Vladimír Vondruš <mosra@centrum.cz>
 
     Permission is hereby granted, free of charge, to any person obtaining a
@@ -66,11 +66,10 @@ for more involved usage, usage of non-default or multiple framebuffers.
 
 See also @ref GL-AbstractFramebuffer-performance-optimization "relevant section in AbstractFramebuffer".
 
-If either @gl_extension{ARB,direct_state_access} (part of OpenGL 4.5) or
-@gl_extension{EXT,direct_state_access} desktop extension is available, functions
-@ref checkStatus(), @ref mapForDraw(), @ref mapForRead() and @ref invalidate()
-use DSA to avoid unnecessary calls to @fn_gl{BindFramebuffer}. See their
-respective documentation for more information.
+If @gl_extension{ARB,direct_state_access} (part of OpenGL 4.5) is available,
+functions @ref checkStatus(), @ref mapForDraw(), @ref mapForRead() and
+@ref invalidate() use DSA to avoid unnecessary calls to @fn_gl{BindFramebuffer}.
+See their respective documentation for more information.
 */
 class MAGNUM_GL_EXPORT DefaultFramebuffer: public AbstractFramebuffer {
     friend Context;
@@ -164,8 +163,8 @@ class MAGNUM_GL_EXPORT DefaultFramebuffer: public AbstractFramebuffer {
          *
          * @see @ref mapForRead()
          * @m_enum_values_as_keywords
-         * @requires_gles30 Extension @gl_extension2{NV,read_buffer,GL_NV_read_buffer}
-         *      in OpenGL ES 2.0.
+         * @requires_gles30 Extension @gl_extension{NV,read_buffer} in OpenGL
+         *      ES 2.0.
          * @requires_webgl20 Framebuffer read mapping is not available in WebGL
          *      1.0.
          */
@@ -222,7 +221,7 @@ class MAGNUM_GL_EXPORT DefaultFramebuffer: public AbstractFramebuffer {
 
             /**
              * Read from front buffer.
-             * @requires_es_extension Extension @gl_extension2{NV,read_buffer_front,GL_NV_read_buffer}
+             * @requires_es_extension Extension @gl_extension2{NV,read_buffer_front,NV_read_buffer}
              * @requires_gles Reading from front buffer is not available in
              *      WebGL.
              */
@@ -306,7 +305,13 @@ class MAGNUM_GL_EXPORT DefaultFramebuffer: public AbstractFramebuffer {
         };
         #endif
 
-        explicit MAGNUM_GL_LOCAL DefaultFramebuffer();
+        /**
+         * @brief Constructor
+         *
+         * Not meant to be constructed on the application side, use the
+         * @ref GL::defaultFramebuffer instance directly.
+         */
+        constexpr explicit DefaultFramebuffer(): AbstractFramebuffer{0, {}, ObjectFlag::Created|ObjectFlag::DeleteOnDestruction} {}
 
         /** @brief Copying is not allowed */
         DefaultFramebuffer(const DefaultFramebuffer&) = delete;
@@ -324,9 +329,8 @@ class MAGNUM_GL_EXPORT DefaultFramebuffer: public AbstractFramebuffer {
          * @brief Check framebuffer status
          * @param target    Target for which to check the status
          *
-         * If neither @gl_extension{ARB,direct_state_access} (part of OpenGL 4.5)
-         * nor @gl_extension{EXT,direct_state_access} desktop extension is
-         * available, the framebuffer is bound before the operation (if not
+         * If @gl_extension{ARB,direct_state_access} (part of OpenGL 4.5) is
+         * not available, the framebuffer is bound before the operation (if not
          * already).
          *
          * The @p target parameter is ignored on OpenGL ES 2.0 if none of
@@ -334,7 +338,6 @@ class MAGNUM_GL_EXPORT DefaultFramebuffer: public AbstractFramebuffer {
          * or @gl_extension{NV,framebuffer_blit} is available and also on WebGL
          * 1.0.
          * @see @fn_gl2_keyword{CheckNamedFramebufferStatus,CheckFramebufferStatus},
-         *      @fn_gl_extension_keyword{CheckNamedFramebufferStatus,EXT,direct_state_access},
          *      eventually @fn_gl{BindFramebuffer} and @fn_gl_keyword{CheckFramebufferStatus}
          * @requires_gl30 Extension @gl_extension{ARB,framebuffer_object}
          */
@@ -373,13 +376,11 @@ class MAGNUM_GL_EXPORT DefaultFramebuffer: public AbstractFramebuffer {
          *
          * @snippet MagnumGL.cpp DefaultFramebuffer-usage-map
          *
-         * If neither @gl_extension{ARB,direct_state_access} (part of OpenGL 4.5)
-         * nor @gl_extension{EXT,direct_state_access} desktop extension is
-         * available, the framebuffer is bound before the operation (if not
+         * If @gl_extension{ARB,direct_state_access} (part of OpenGL 4.5) is
+         * not available, the framebuffer is bound before the operation (if not
          * already).
          * @see @ref maxDrawBuffers(), @ref maxDualSourceDrawBuffers(),
          *      @ref mapForRead(), @fn_gl2_keyword{NamedFramebufferDrawBuffers,DrawBuffers},
-         *      @fn_gl_extension_keyword{FramebufferDrawBuffers,EXT,direct_state_access},
          *      eventually @fn_gl{BindFramebuffer} and @fn_gl_keyword{DrawBuffers}
          * @requires_gles30 Extension @gl_extension{EXT,draw_buffers} in OpenGL
          *      ES 2.0.
@@ -396,12 +397,10 @@ class MAGNUM_GL_EXPORT DefaultFramebuffer: public AbstractFramebuffer {
          * Similar to above function, can be used in cases when shader has
          * only one (unnamed) output.
          *
-         * If neither @gl_extension{ARB,direct_state_access} (part of OpenGL 4.5)
-         * nor @gl_extension{EXT,direct_state_access} desktop extension is
-         * available, the framebuffer is bound before the operation (if not
+         * If @gl_extension{ARB,direct_state_access} (part of OpenGL 4.5) is
+         * not available, the framebuffer is bound before the operation (if not
          * already).
          * @see @ref mapForRead(), @fn_gl2_keyword{NamedFramebufferDrawBuffer,DrawBuffer},
-         *      @fn_gl_extension_keyword{FramebufferDrawBuffer,EXT,direct_state_access},
          *      eventually @fn_gl{BindFramebuffer} and @fn_gl_keyword{DrawBuffer}
          *      or @fn_gl{DrawBuffers} in OpenGL ES 3.0
          * @requires_gles30 Extension @gl_extension{EXT,draw_buffers} in OpenGL
@@ -417,15 +416,13 @@ class MAGNUM_GL_EXPORT DefaultFramebuffer: public AbstractFramebuffer {
          * @param attachment        Buffer attachment
          * @return Reference to self (for method chaining)
          *
-         * If neither @gl_extension{ARB,direct_state_access} (part of OpenGL 4.5)
-         * nor @gl_extension{EXT,direct_state_access} desktop extension is
-         * available, the framebuffer is bound before the operation (if not
+         * If @gl_extension{ARB,direct_state_access} (part of OpenGL 4.5) is
+         * not available, the framebuffer is bound before the operation (if not
          * already).
          * @see @ref mapForDraw(), @fn_gl2_keyword{NamedFramebufferReadBuffer,ReadBuffer},
-         *      @fn_gl_extension_keyword{FramebufferReadBuffer,EXT,direct_state_access},
          *      eventually @fn_gl{BindFramebuffer} and @fn_gl_keyword{ReadBuffer}
-         * @requires_gles30 Extension @gl_extension2{NV,read_buffer,GL_NV_read_buffer}
-         *      in OpenGL ES 2.0.
+         * @requires_gles30 Extension @gl_extension{NV,read_buffer} in OpenGL
+         *      ES 2.0.
          * @requires_webgl20 Framebuffer read mapping is not available in WebGL
          *      1.0.
          */
@@ -504,27 +501,14 @@ class MAGNUM_GL_EXPORT DefaultFramebuffer: public AbstractFramebuffer {
 };
 
 /** @brief Default framebuffer instance */
+/* Even though the constructor is constexpr, this variable can't -- all
+   framebuffer APIs are non-const since they modify global GL state and besides
+   that we also need to modify its private _viewport member quite a lot */
 extern DefaultFramebuffer MAGNUM_GL_EXPORT defaultFramebuffer;
 
 /** @debugoperatorclassenum{DefaultFramebuffer,DefaultFramebuffer::Status} */
 MAGNUM_GL_EXPORT Debug& operator<<(Debug& debug, DefaultFramebuffer::Status value);
 
-}
-
-#ifdef MAGNUM_BUILD_DEPRECATED
-/* Note: needs to be prefixed with Magnum:: otherwise Doxygen can't find it */
-
-/** @brief @copybrief GL::DefaultFramebuffer
- * @deprecated Use @ref GL::DefaultFramebuffer instead.
- */
-typedef CORRADE_DEPRECATED("use GL::DefaultFramebuffer instead") Magnum::GL::DefaultFramebuffer DefaultFramebuffer;
-
-/** @brief @copybrief GL::defaultFramebuffer
- * @deprecated Use @ref GL::defaultFramebuffer instead.
- */
-extern CORRADE_DEPRECATED("use GL::defaultFramebuffer instead") MAGNUM_EXPORT Magnum::GL::DefaultFramebuffer& defaultFramebuffer;
-#endif
-
-}
+}}
 
 #endif
