@@ -3,7 +3,7 @@
 /*
     This file is part of Magnum.
 
-    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018
+    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019
               Vladimír Vondruš <mosra@centrum.cz>
 
     Permission is hereby granted, free of charge, to any person obtaining a
@@ -31,7 +31,9 @@
 
 #include <Corrade/Containers/Array.h>
 
-#include "Magnum/ImageView.h"
+#include "Magnum/DimensionTraits.h"
+#include "Magnum/PixelStorage.h"
+#include "Magnum/Trade/Trade.h"
 #include "Magnum/Trade/visibility.h"
 
 namespace Magnum { namespace Trade {
@@ -74,7 +76,8 @@ compressed properties through @ref compressedStorage() and
 
 @snippet MagnumTrade.cpp ImageData-usage
 
-@see @ref ImageData1D, @ref ImageData2D, @ref ImageData3D
+@see @ref ImageData1D, @ref ImageData2D, @ref ImageData3D,
+    @ref Image-pixel-views
 */
 template<UnsignedInt dimensions> class ImageData {
     public:
@@ -165,22 +168,6 @@ template<UnsignedInt dimensions> class ImageData {
          */
         template<class T> explicit ImageData(PixelStorage storage, T format, const VectorTypeFor<dimensions, Int>& size, Containers::Array<char>&& data, const void* importerState = nullptr) noexcept;
 
-        #if defined(MAGNUM_BUILD_DEPRECATED) && defined(MAGNUM_TARGET_GL)
-        /**
-         * @brief Construct uncompressed image data
-         * @param format            Format of pixel data
-         * @param type              Data type of pixel data
-         * @param size              Image size
-         * @param data              Image data
-         * @param importerState     Importer-specific state
-         *
-         * @deprecated Use the generic @ref ImageData(PixelStorage, T, U, const VectorTypeFor<dimensions, Int>&, Containers::Array<char>&&, const void*)
-         *      instead --- the pixel storage parameter is used to distinguish
-         *      between compressed and uncompressed data.
-         */
-        template<class = void> explicit CORRADE_DEPRECATED("use Image(PixelStorage, T, U, const VectorTypeFor<dimensions, Int>&, Containers::Array<char>&&, const void*) instead") ImageData(PixelFormat format, GL::PixelType type, const VectorTypeFor<dimensions, Int>& size, Containers::Array<char>&& data, const void* importerState = nullptr) noexcept;
-        #endif
-
         /**
          * @brief Construct compressed image data
          * @param storage           Storage of compressed pixel data
@@ -250,7 +237,9 @@ template<UnsignedInt dimensions> class ImageData {
          */
         /* Not restricted to const&, because we might want to pass the view to
            another function in an oneliner (e.g. saving screenshot) */
-        /*implicit*/ operator ImageView<dimensions>() const;
+        /*implicit*/ operator BasicMutableImageView<dimensions>();
+        /** @overload */
+        /*implicit*/ operator BasicImageView<dimensions>() const;
 
         /**
          * @brief Conversion to compressed view
@@ -260,7 +249,9 @@ template<UnsignedInt dimensions> class ImageData {
          */
         /* Not restricted to const&, because we might want to pass the view to
            another function in an oneliner (e.g. saving screenshot) */
-        /*implicit*/ operator CompressedImageView<dimensions>() const;
+        /*implicit*/ operator BasicMutableCompressedImageView<dimensions>();
+        /** @overload */
+        /*implicit*/ operator BasicCompressedImageView<dimensions>() const;
 
         /**
          * @brief Storage of pixel data
@@ -297,15 +288,6 @@ template<UnsignedInt dimensions> class ImageData {
          */
         UnsignedInt formatExtra() const;
 
-        #if defined(MAGNUM_BUILD_DEPRECATED) && defined(MAGNUM_TARGET_GL)
-        /**
-         * @brief Data type of pixel data
-         *
-         * @deprecated Cast @ref formatExtra() to @ref GL::PixelType instead.
-         */
-        CORRADE_DEPRECATED("cast formatExtra() to GL::PixelType instead") GL::PixelType type() const { return GL::PixelType(formatExtra()); } /* LCOV_EXCL_LINE */
-        #endif
-
         /**
          * @brief Storage of compressed pixel data
          *
@@ -353,38 +335,81 @@ template<UnsignedInt dimensions> class ImageData {
            needless state changes -- thus the calculation can't be done */
 
         /**
-         * @brief Raw data
+         * @brief Image data
          *
-         * @see @ref release()
+         * @see @ref release(), @ref pixels()
          */
-        Containers::ArrayView<char> data()
-            #ifndef CORRADE_GCC47_COMPATIBILITY
-            &
-            #endif
-            { return _data; }
-        #ifndef CORRADE_GCC47_COMPATIBILITY
-        Containers::ArrayView<char> data() && = delete; /**< @overload */
-        #endif
+        Containers::ArrayView<char> data() & { return _data; }
 
         /** @overload */
-        Containers::ArrayView<const char> data() const
-            #ifndef CORRADE_GCC47_COMPATIBILITY
-            &
-            #endif
-            { return _data; }
-        #ifndef CORRADE_GCC47_COMPATIBILITY
-        Containers::ArrayView<const char> data() const && = delete; /**< @overload */
-        #endif
+        Containers::ArrayView<const char> data() const & { return _data; }
 
-        /** @overload */
-        template<class T> T* data() {
+        /**
+         * @brief Image data from a r-value
+         *
+         * Unlike @ref data(), which returns a view, this is equivalent to
+         * @ref release() to avoid a dangling view when the temporary instance
+         * goes out of scope.
+         * @todoc stupid doxygen can't link to & overloads ffs
+         */
+        Containers::Array<char> data() && { return release(); }
+
+        /** @overload
+         * @todo what to do here?!
+         */
+        Containers::Array<char> data() const && = delete;
+
+        #ifdef MAGNUM_BUILD_DEPRECATED
+        /**
+         * @brief Image data in a particular type
+         * @deprecated Use non-templated @ref data() together with
+         *      @ref Corrade::Containers::arrayCast() instead for properly
+         *      bounds-checked type conversion.
+         */
+        template<class T> CORRADE_DEPRECATED("use data() together with Containers::arrayCast() instead") T* data() {
             return reinterpret_cast<T*>(_data.data());
         }
 
-        /** @overload */
-        template<class T> const T* data() const {
+        /**
+         * @brief Image data in a particular type
+         * @deprecated Use non-templated @ref data() together with
+         *      @ref Corrade::Containers::arrayCast() instead for properly
+         *      bounds-checked type conversion.
+         */
+        template<class T> CORRADE_DEPRECATED("use data() together with Containers::arrayCast() instead") const T* data() const {
             return reinterpret_cast<const T*>(_data.data());
         }
+        #endif
+
+        /**
+         * @brief View on pixel data
+         *
+         * Provides direct and easy-to-use access to image pixels. Expects that
+         * the image is not compressed. See @ref Image-pixel-views for more
+         * information.
+         * @see @ref isCompressed()
+         */
+        Containers::StridedArrayView<dimensions + 1, char> pixels();
+        Containers::StridedArrayView<dimensions + 1, const char> pixels() const; /**< @overload */
+
+        /**
+         * @brief View on pixel data with a concrete pixel type
+         *
+         * Compared to non-templated @ref pixels() in addition casts the pixel
+         * data to a specified type. The user is responsible for choosing
+         * correct type for given @ref format() --- checking it on the library
+         * side is not possible for the general case.
+         */
+        template<class T> Containers::StridedArrayView<dimensions, T> pixels() {
+            /* Deliberately not adding a StridedArrayView include, it should
+               work without since this is a templated function */
+            return Containers::arrayCast<dimensions, T>(pixels());
+        }
+
+        /** @overload */
+        template<class T> Containers::StridedArrayView<dimensions, const T> pixels() const {
+            return Containers::arrayCast<dimensions, const T>(pixels());
+        }
 
         /**
          * @brief Release data storage
@@ -403,6 +428,11 @@ template<UnsignedInt dimensions> class ImageData {
         const void* importerState() const { return _importerState; }
 
     private:
+        /* For custom deleter checks. Not done in the constructors here because
+           the restriction is pointless when used outside of plugin
+           implementations. */
+        friend AbstractImporter;
+
         explicit ImageData(CompressedPixelStorage storage, UnsignedInt format, const VectorTypeFor<dimensions, Int>& size, Containers::Array<char>&& data, const void* importerState = nullptr) noexcept;
 
         bool _compressed;
@@ -421,32 +451,20 @@ template<UnsignedInt dimensions> class ImageData {
         const void* _importerState;
 };
 
-/** @brief One-dimensional image */
+/** @brief One-dimensional image data */
 typedef ImageData<1> ImageData1D;
 
-/** @brief Two-dimensional image */
+/** @brief Two-dimensional image data */
 typedef ImageData<2> ImageData2D;
 
-/** @brief Three-dimensional image */
+/** @brief Three-dimensional image data */
 typedef ImageData<3> ImageData3D;
 
-template<UnsignedInt dimensions> template<class T, class U> ImageData<dimensions>::ImageData(const PixelStorage storage, const T format, const U formatExtra, const VectorTypeFor<dimensions, Int>& size, Containers::Array<char>&& data, const void* const importerState) noexcept: ImageData{storage,
-    #if defined(MAGNUM_BUILD_DEPRECATED) && defined(MAGNUM_TARGET_GL)
-    Magnum::Implementation::wrapPixelFormatIfNotGLSpecific(format),
-    #else
-    UnsignedInt(format),
-    #endif
-    UnsignedInt(formatExtra), Magnum::Implementation::pixelSizeAdl(format, formatExtra), size, std::move(data), importerState} {
+template<UnsignedInt dimensions> template<class T, class U> ImageData<dimensions>::ImageData(const PixelStorage storage, const T format, const U formatExtra, const VectorTypeFor<dimensions, Int>& size, Containers::Array<char>&& data, const void* const importerState) noexcept: ImageData{storage, UnsignedInt(format), UnsignedInt(formatExtra), Magnum::Implementation::pixelSizeAdl(format, formatExtra), size, std::move(data), importerState} {
     static_assert(sizeof(T) <= 4 && sizeof(U) <= 4,
         "format types larger than 32bits are not supported");
 }
 
-#if defined(MAGNUM_BUILD_DEPRECATED) && defined(MAGNUM_TARGET_GL)
-CORRADE_IGNORE_DEPRECATED_PUSH
-template<UnsignedInt dimensions> template<class> inline ImageData<dimensions>::ImageData(const PixelFormat format, const GL::PixelType type, const VectorTypeFor<dimensions, Int>& size, Containers::Array<char>&& data, const void* const importerState) noexcept: ImageData{PixelStorage{}, format, type, size, std::move(data), importerState} {}
-CORRADE_IGNORE_DEPRECATED_POP
-#endif
-
 template<UnsignedInt dimensions> template<class T> ImageData<dimensions>::ImageData(const PixelStorage storage, const T format, const VectorTypeFor<dimensions, Int>& size, Containers::Array<char>&& data, const void* const importerState) noexcept: ImageData{storage, UnsignedInt(format), {}, Magnum::Implementation::pixelSizeAdl(format), size, std::move(data), importerState} {
     static_assert(sizeof(T) <= 4,
         "format types larger than 32bits are not supported");
@@ -457,46 +475,6 @@ template<UnsignedInt dimensions> template<class T> ImageData<dimensions>::ImageD
         "format types larger than 32bits are not supported");
 }
 
-template<UnsignedInt dimensions> inline ImageData<dimensions>::ImageData(ImageData<dimensions>&& other) noexcept: _compressed{std::move(other._compressed)}, _size{std::move(other._size)}, _data{std::move(other._data)}, _importerState{std::move(other._importerState)} {
-    if(_compressed) {
-        new(&_compressedStorage) CompressedPixelStorage{std::move(other._compressedStorage)};
-        _compressedFormat = std::move(other._compressedFormat);
-    }
-    else {
-        new(&_storage) PixelStorage{std::move(other._storage)};
-        _format = std::move(other._format);
-        _formatExtra = std::move(other._formatExtra);
-        _pixelSize = std::move(other._pixelSize);
-    }
-
-    other._size = {};
-}
-
-template<UnsignedInt dimensions> inline ImageData<dimensions>& ImageData<dimensions>::operator=(ImageData<dimensions>&& other) noexcept {
-    using std::swap;
-    swap(_compressed, other._compressed);
-    if(_compressed) {
-        swap(_compressedStorage, other._compressedStorage);
-        swap(_compressedFormat, other._compressedFormat);
-    }
-    else {
-        swap(_storage, other._storage);
-        swap(_format, other._format);
-    }
-    swap(_formatExtra, other._formatExtra);
-    swap(_pixelSize, other._pixelSize);
-    swap(_size, other._size);
-    swap(_data, other._data);
-    swap(_importerState, other._importerState);
-    return *this;
-}
-
-template<UnsignedInt dimensions> inline Containers::Array<char> ImageData<dimensions>::release() {
-    Containers::Array<char> data{std::move(_data)};
-    _size = {};
-    return data;
-}
-
 }}
 
 #endif
